//
//  konyWebSocketCallbacks.h
//  KonyWebSocket
//
//  Created by Sai Sanjeev on 1/1/2020.
//  Copyright © 2020 Kony. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

typedef enum
    {
        onMessage = 0,
        onError,
        onClose
    } CallbackType;

@interface KonyWebSocketCallbacks : NSObject

/**
 * Method to be implemented for handling on message callback
 *
 * @param result object representing the result
 */
typedef void (^onMessageCallback) (NSString* result);

/**
 * Method to be implemented for handling on error callback
 *
 * @param error object representing the error(s)
 */
typedef void (^onErrorCallback) (NSString* error);

/**
 * Method to be implemented for handling on close callback
 *
 * @param result object representing the result
 */
typedef void (^onCloseCallback) (NSString* result);

+ (instancetype) getSharedInstance;

- (void) setOnMessageCallback: (onMessageCallback)onMessageCallback;

- (void) setOnErrorCallback: (onErrorCallback)onErrorCallback;

- (void) setOnCloseCallback: (onCloseCallback)onCloseCallback;

- (void) executeWithCallback: (CallbackType)callbackType result: (id)result;

@end

NS_ASSUME_NONNULL_END
