//
//  IKonyWebSocket.h
//  KonyWebSocket
//
//  Created by Sai Sanjeev on 1/1/2020.
//  Copyright © 2020 Kony. All rights reserved.
//

#import <SocketRocket/SocketRocket.h>
#import "KonyWebSocketCallbacks.h"

@interface KonyWebSocket : NSObject

- (instancetype) init;

/**
 Publish events to the server
 
 @param eventsJson           eventsJson string to be published to the server - NSString
                    
                       example :
                       "{"events" : [{"topic" : "transaction/deposit",
                       "data" : { "amount": "1500",
                       "user": "clientevents",
                       "account": "1000",
                       "transaction": "deposit" }}]}"
 
 @param publishOptions  additional options for publishing - NSDictionary
 */
- (void) publishServerEvents: (NSString *) eventsJson
              publishOptions: (NSDictionary *) publishOptions;

/**
 Subscribe for server events
 
 @param url                               server url for web socket
 @param eventsJson                eventsJson string to be published to the server
 
                         example :
                         "{"subscribe" : ["transaction/deposit",
                         "transaction/withdraw"]}"
 
 @param onMessage                  onMessage callback to receive messages
 @param onError                      onError callback
 @param subscribeOptions  additional options for subscribing - NSDictionary
 */
- (void) subscribeServerEvents: (NSString *) url
                  eventsToSend: (NSString *) eventsJson
                     onMessage: (onMessageCallback) onMessage
                       onError: (onErrorCallback) onError
              subscribeOptions: (NSDictionary *) subscribeOptions;

/**
 UnSubscribe from server events
 
 @param eventsJson                    eventsJson string to unsubscribe - NSString
                           example :
                           "{"unsubscribe" : ["transaction/deposit"]}"
 
 @param onClose                           onClose for closing the websocket - optional
 @param unSubscribeOptions   additional options to unsubscribe - NSDictionary
 */
- (void) unSubscribeServerEvents: (NSString *) eventsJson
                 onCloseCallback: (onCloseCallback) onClose
              unSubscribeOptions: (NSDictionary *) unSubscribeOptions;

@end
